#!/bin/bash
#
# Mapserver Vulnerability checker (http://www.mapserver.org) 
#
# vulnerable versions (a regex)
VULN_VERSIONS="5\. 6\. 4\."
PATCH_VERSIONS="6\.2"

WORKAROUND_URL="http://www.mapserver.org"

strings /bin/ls > /dev/null 2>&1
DONT_HAVE_STRINGS=$?
HAVE_WFS=0
HAVE_POSTGRES=0

function about {
	echo "Mapserver Vulnerability checker (http://www.mapserver.org)"
}
function usage {
	echo "Usage: $0 path/to/mapserv"
}

about

if [ "$1" != "" ]; then
	MAPSERVER=$1
	MAPSERVER_VERSION=`${MAPSERVER} -v | awk '{print $3}'`

	for vv in ${PATCH_VERSIONS} ; do
		if [[ "${MAPSERVER_VERSION}" =~ $vv ]]; then
			echo "Congratulations, you are running a safe version (${MAPSERVER_VERSION})!"
			exit 0
		fi
	done

	if [ -x "$MAPSERVER" ]; then
		if [ "${DONT_HAVE_STRINGS=}" == 0 ]; then
			# we do have strings
			strings ${MAPSERVER} | grep -q PropertyIsEqualTo > /dev/null 2>&1
			HAVE_WFS=$?
			strings ${MAPSERVER} | grep -q postgis > /dev/null 2>&1
			HAVE_POSTGRES=$?
		else
			echo "strings executable not availble, proceeding without it"
			echo "detection will be less accurate"
			${MAPSERVER} -v | grep -v -q WFS > /dev/null 2>&1
			HAVE_WFS=$?
			${MAPSERVER} -v | grep -v -q POSTG > /dev/null 2>&1
			HAVE_POSTGRES=$?
		fi
		echo "WFS Services     :   ${HAVE_WFS}"
		echo "POSTGIS Services :   ${HAVE_POSTGRES}"
		echo "Mapserver version:   ${MAPSERVER_VERSION}"
		echo ""
		if [ "${HAVE_WFS}" == "0" ] && [ "${HAVE_POSTGRES}" == "0" ]; then
			for v in ${VULN_VERSIONS} ; do
				if [[ "${MAPSERVER_VERSION}" =~ $v ]]; then
					echo "We are sorry, but you have a vulnerable version."
					echo "You are urged to upgrade or take the mitigating actions"
					echo "described in: ${WORKAROUND_URL}"
				fi
			done
		else
			for v in ${VULN_VERSIONS} ; do
                                if [[ "${MAPSERVER_VERSION}" =~ $v ]]; then
                                        echo "You have a vulnerable version, but have not enabled both postgis"
                                        echo "and WFS services. You are therefore not at immediate risk."
                                        echo "We suggest anyway to upgrade or take the mitigating actions"
                                        echo "described in: ${WORKAROUND_URL}"
                                fi
                        done
		fi
	else
		echo "Error: $MAPSERVER not found or not executable."
	fi
else
	usage
fi


